require "./lib/vocabularies/rdms"
# Generated via
#  `rails generate hyrax:work Dataset`
class Dataset < ActiveFedora::Base
  include ::Hyrax::WorkBehavior
  include ::Hyrax::TombstoneBehavior
  self.indexer = DatasetIndexer
  # Change this to restrict which works can be added as a child.
  # self.valid_child_concerns = []
  validates :title, presence: { message: 'Your dataset must have a title.' }

  after_create :set_default_tombstone_status
  # ------ properties from core metadata ------
  # property date_modified          - not displayed (filled in by the system)
  # property date_uploaded          - not displayed (filled in by the system)
  # property depositor              - not displayed (filled in by the system)
  # property title                  - keep

  # ------ properties from basic metadata ------
  # property alternative_title      - keep
  # property label                  - not displayed, used for file version label
  # property relative_path          - not displayed, used for file storage
  # property import_url             - not displayed, used for file imports
  # property resource_type          - keep
  # property creator                - do not display
  # property contributor            - do not display
  # property description            - keep
  # property abstract               - keep
  # property keyword                - keep
  # property license                - keep
  # property rights_notes           - do not display
  # property rights_statement       - do not display
  # property access_right           - do not display
  # property publisher              - keep
  # property date_created           - do not display
  # property subject                - keep
  # property language               - keep
  # property identifier             - do not display (used to store alternate identifiers)
  # property based_near             - do not display
  # property related_url            - do not display
  # property bibliographic_citation - not displayed (generated by the system)
  # property source                 - do not display (should be filled in by the system for bulk imports)

  property :doi, predicate: ::RDF::Vocab::DataCite.doi, multiple: false do |index|
    index.as :symbol
  end

  property :tombstone_status, predicate: ::RDF::Vocab::Rdms.tombstoneStatus, multiple: false do |index|
    index.as :symbol
  end

  property :tombstone_date, predicate: ::RDF::Vocab::Rdms.tombstoneDate, multiple: false do |index|
    index.as :symbol
  end

  property :is_tombstoned, predicate: ::RDF::Vocab::Rdms.isTombstone, multiple: false do |index|
    index.as :symbol
  end

  property :software_version, predicate: ::RDF::Vocab::Rdms.softwareVersion do |index|
    index.as :stored_searchable
  end

  property :dataset_method, predicate: ::RDF::Vocab::Rdms.datasetMethod do |index|
    index.as :stored_searchable
  end

  property :series_information, predicate: ::RDF::Vocab::Rdms.seriesInformation do |index|
    index.as :stored_searchable
  end

  property :table_of_contents, predicate: ::RDF::Vocab::Rdms.tableOfContents do |index|
    index.as :stored_searchable
  end

  property :technical_information, predicate: ::RDF::Vocab::Rdms.technicalInformation do |index|
    index.as :stored_searchable
  end

  property :complex_person, predicate: ::RDF::Vocab::SIOC.has_creator, class_name:"ComplexPerson"

  # property :date_published, predicate: ::RDF::Vocab::Rdms.datePublished, multiple: false do |index|
  #   index.type :date
  #   index.as :stored_sortable
  # end

  property :complex_date, predicate: ::RDF::Vocab::DC.date, class_name:"ComplexDate"

  property :complex_identifier, predicate: ::RDF::Vocab::Rdms.identifier, class_name:"ComplexIdentifier"

  # size and format to be obtained from the files attached

  # description and abstract is in the basic metadata. Ignoring other descriptions for now.

  # Geolocation - would this be used? Ignoring for now.

  property :complex_funding_reference, predicate: ::RDF::Vocab::DataCite.fundref, class_name:"ComplexFundingReference"

  property :complex_relation, predicate: ::RDF::Vocab::DC.relation, class_name:"ComplexRelation"

  # Required due to bug saving nested resources
  property :updated_subresources, predicate: ::RDF::Vocab::Rdms.updatedSubresources, class_name: "ActiveTriples::Resource"

  # This must be included at the end, because it finalizes the metadata
  # schema (by adding accepts_nested_attributes)
  include ::Hyrax::BasicMetadata
  include ComplexValidation
  accepts_nested_attributes_for :complex_person, reject_if: :person_blank, allow_destroy: true
  accepts_nested_attributes_for :complex_date, reject_if: :date_blank, allow_destroy: true
  accepts_nested_attributes_for :complex_identifier, reject_if: :identifier_blank, allow_destroy: true
  accepts_nested_attributes_for :complex_funding_reference, reject_if: :fundref_blank, allow_destroy: true
  accepts_nested_attributes_for :complex_relation, reject_if: :relation_blank, allow_destroy: true
  accepts_nested_attributes_for :updated_subresources, allow_destroy: true

  private

  def set_default_tombstone_status
    self.tombstone_status = false
    self.save
  end
  
end
